<?php
namespace BuyPlayTix\Wordpress;
class Plugin {

  public static function tickets_shorttag($attributes) {
    $oauth = new OAuth();
    $producer = $oauth->getProducer();
    $producer_url_name = $producer->url_name;
    
    $production_url = $producer->secure_path . "/reservewidget/" . $attributes["url"] . ".html";
    $contents = file_get_contents($production_url);
    if(strstr($contents, '404: Page Not Found') === FALSE) {
      return '<script type="text/javascript">' . $contents . '</script>';
    }
    return '<div class="error">Error: Production ' . $attributes["url"] . ' not found.</div>';
  }

  public static function donate_shorttag($attributes) {
    $oauth = new OAuth();
    $producer = $oauth->getProducer();
    wp_enqueue_style(
      'bpt-ecss.css',
      $producer->css_path . '/ecss.css'
    );
    wp_enqueue_script(
      'bpt-common',
      plugins_url('/buyplaytix/js/common.js'),
      array( 'jquery' )
    );
    wp_enqueue_script(
      'bpt-transaction-common',
      plugins_url('/buyplaytix/js/transaction-common.js'),
      array( 'jquery' )
    );
    wp_enqueue_script(
      'bpt-donate',
      plugins_url('/buyplaytix/js/donate.js'),
      array( 'jquery' )
    );
    
    
    return Plugin::render("donate", array(
      "producer" => $producer
    ));
  }
  

  public static function calendar_shorttag($attributes) {
    $oauth = new OAuth();
    $producer = $oauth->getProducer();
    wp_enqueue_script(
      'fullcalendar',
      plugins_url('/buyplaytix/js/vendor/fullcalendar/fullcalendar.min.js'),
      array( 'jquery' )
    );
    wp_enqueue_script(
      'fullcalendar.bpt',
      plugins_url('/buyplaytix/js/vendor/fullcalendar/bpt.js'),
      array( 'jquery' )
    );
    wp_enqueue_style(
      'fullcalendar.css',
      plugins_url('/buyplaytix/css/vendor/fullcalendar/fullcalendar.css')
    );
    wp_enqueue_style(
    'custom.css',
    $producer->css_path . '/customcss.css?wrapper=div.calendar-wrapper'
    );
    
    
    
    return Plugin::render("calendar-large", array(
    "producer" => $producer
    ));
  }
  
  public function getCalendarUrl($replacements = array()) {
    $attributes = $this->getAttributes();
    $currentValues = array(
        "view" => $attributes["view"],
        "previous" => "false",
        "next" => "false",
        "current" =>   $attributes["current"],
        "metro" =>   $attributes["metro"],
    );
    foreach($replacements as $key => $value) {
      $currentValues[$key] = $value;
    }
    return "calendar.html?" . http_build_query($currentValues);
  }

  public static function admin_menu() {
    add_options_page('BuyPlayTix Options', 'BuyPlayTix', 'manage_options', 'buyplaytix-manage-options', array('BuyPlayTix\Wordpress\Plugin', 'admin_options'));
  }

  public static function admin_notices() {
    if(!OAuth::isConfigured()) {
      $url = site_url("wp-admin/options-general.php?page=buyplaytix-manage-options");
      echo "<div id='notice' class='updated fade'><p>BuyPlayTix Plugin is not configured yet. Please <a href='" . $url . "'>do it now</a>.</p></div>\n";
    }
  }

  public static function admin_options() {
    if ( !current_user_can( 'manage_options' ) )  {
      wp_die( __( 'You do not have sufficient permissions to access this page.' ) );
    }
    
    wp_enqueue_script(
      'bpt-common',
      plugins_url('/buyplaytix/js/common.js'),
      array( 'jquery' )
    );
    wp_enqueue_script(
      'bpt-wordpress',
      plugins_url('/buyplaytix/js/wordpress.js'),
      array( 'jquery' )
    );
    
    $oauth = new OAuth();
    $productions = $oauth->getProductions();
    $producer = $oauth->getProducer();
    
    $pages = array_merge(get_posts(), get_pages());
    //$all_pages = array_merge($pages, get_pages(array("post_type" => "post")));
    print Plugin::render("admin-oauth", array(
      "productions" => $productions,
      "producer" => $producer,
      "pages" => $pages,
    ));
  }

  public static function admin_init() {

    if($_REQUEST["page"] != "buyplaytix-manage-options") {
      return;
    }

    $action = $_REQUEST["action"];
    if($action == "reauthorize") {
      update_option("buyplaytix_token", "");
      update_option("buyplaytix_secret", "");
    }
    elseif($action == "linkProduction") {
      $oauth = new OAuth();
      $response = $oauth->linkProductions($_REQUEST["productions"], $_REQUEST["pages"]);
      if($response->code != 200) {
        add_settings_error("BuyPlayTix", "linkProduction", "Unable to link production.");
      }
      return;
    }

    $oauth = new OAuth();
    $oauth->finish();
    
    if(!OAuth::isConfigured()) {
      $oauth->signin(site_url("wp-admin/options-general.php?page=buyplaytix-manage-options"));
    }
  }

  private static function render($page, $attributes = array()) {
    \HTML_Template_Nest_View::$CACHE = false;
    \HTML_Template_Nest_View::$INCLUDE_PATHS = array(realpath(dirname(__FILE__) . "/../../../views"));
    
    $view = new \HTML_Template_Nest_View($page);
    $view->setAttributes($attributes);
    return $view->render();
  }

  public static function settings_link($links, $file) {
 
    if ( $file == 'buyplaytix/buyplaytix.php' ) {
        /* Insert the link at the end*/
        $links['settings'] = sprintf( '<a href="%s"> %s </a>', admin_url( 'options-general.php?page=buyplaytix-manage-options' ), __( 'Settings', 'plugin_domain' ) );
    }
    return $links;
  }
}
